==> crow-2.15.0.tar.bz2: crow-2.15.0/crow/session.h <==
/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005-2009 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person
 *  obtaining a copy of this software and associated documentation
 *  files (the "Software"), to deal in the Software without
 *  restriction, including without limitation the rights to use, copy,
 *  modify, merge, publish, distribute, sublicense, and/or sell copies
 *  of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind,
 *  express or implied, including but not limited to the warranties of
 *  merchantability, fitness for a particular purpose and
 *  noninfringement. In no event shall the authors or copyright
 *  holders be liable for any claim, damages or other liability,
 *  whether in an action of contract, tort or otherwise, arising from,
 *  out of or in connection with the software or the use or other
 *  dealings in the software.
 */

#ifndef CROW_SESSION_H
#define CROW_SESSION_H

#include "controller.h"

namespace Crow {

enum SessionOperation { soRemove, soInsert, soUpdate };
enum SessionRole { srNone, srScalar, srVector, srObject };

class Session : public Object {

friend class SessionSupplier;

public:
	static PSession create();

	typedef std::vector<Property *> Properties;
	static SessionRole getRoleForNodes(const Nodes & nodes);
	static SessionRole getRoleForNode(PNode node);
	static Properties findProperties(Controller * controller, const Nodes & nodes);
	static EditorId getEditorId(const Properties & properties);
	static TypeId getPropertyType(const Properties & properties);

	void init(const Nodes & nodes, const IdPath & path, SessionManager * manager);
	void done();

	PEditor getEditor();
	void setEditor(PEditor editor);
	SessionManager * getManager() { return manager; }

	SessionRole getRole() { return role; }
	bool isMultiple();
	std::string getName();

	bool modelEditable();
	bool canEdit();
	bool isModified();
	bool isTouched();
	bool hasToSave();
	bool canDefault();

	bool hasHidden() { return hasFlags(sfHidden); }
	bool hasNoUndo() { return hasFlags(sfNoUndo); }
	bool areWeakLinks() { return haveFlags(sfWeakLink); }
	bool hasLinkOnly() { return hasFlags(sfLinkOnly); }
	bool areTranslatable() { return haveFlags(sfTranslatable); }
	bool allowTopMasters() { return haveFlags(sfAllowTM); }
	bool isElement();
	bool hasMasterLink();
	bool hasVectorWithMasterLink();
	bool areEmptyLinks();
	TypeId getFinalType();
	TypeId getVectorType();
	Any getScalar();
	Glib::ustring getMeta();
	TypeId getObjectBaseType();
	PNode getLinkTarget();
	TypeId getEntityType();
	EditorId getEditorId();
	bool isFuzzy();

	void resetScalar();
	void resetVector();
	void resetObject();
	void setScalarOnly(Any value);
	void setScalarAndMeta(Any value, const Glib::ustring & meta);
	void setObjectAsLink(PNode object);
	void setObjectAsEntity(TypeId type);

	Property * getProperty1();

	const Nodes * getNodes() { return &nodes; }
	PNode getNode1();

	SessionOperation getOperation() { return operation; }
	const IdPath * getPath() { return &path; }

protected:
	Session();

	bool hasFlags(StateFlags sf);
	bool haveFlags(StateFlags sf);

	Properties getProperties();

private:
	SessionRole role;
	IdPath path;
	Nodes nodes;
	PEditor editor;
	SessionManager * manager;
	SessionOperation operation;

};

class SessionSupplier : public sigc::trackable {

public:
	SessionSupplier();
	virtual ~SessionSupplier() { }

	void setManager(SessionManager * m) { manager= m; }

	void updateSessions();
	void removeSessions();

	void editorsLoad();

	void applySettings();
	void storeSettings();

	PSession findSession(const IdPath & path);
	PSession findSession(PNode node);

	typedef sigc::signal<void, PSession> SignalSessionEvent;
	typedef sigc::signal<void, PSession> SignalAfterLoad;
	typedef sigc::signal<void> SignalPopupMenu;

	sigc::connection connect_session_event(SignalSessionEvent::slot_type slot) { return signalSessionEvent.connect(slot); }
	sigc::connection connect_after_load(SignalAfterLoad::slot_type slot) { return signalAfterLoad.connect(slot); }
	sigc::connection connect_popup_menu(SignalPopupMenu::slot_type slot) { return signalPopupMenu.connect(slot); }

protected:
	SessionManager * getManager() { return manager; }
	const Sessions * getSessions() { return &sessions; }

	virtual void findNodes() { }

	void append(PNode cut, PNode node);
	void append(const IdPath & path, PNode node);
	PSession createSession(const Nodes & nodes, const IdPath & path);
	virtual PEditor createEditor(PSession session);
	void stopSession(PSession);

	void sessionEvent(PSession session) { signalSessionEvent.emit(session); }
	void afterLoad(PSession session) { signalAfterLoad.emit(session); }
	void popupMenu() { signalPopupMenu.emit(); }

	typedef std::map<IdPath, Nodes> NodeMap;
	NodeMap * getFound() { return &found; }

private:
	SignalSessionEvent signalSessionEvent;
	SignalAfterLoad signalAfterLoad;
	SignalPopupMenu signalPopupMenu;
	Sessions sessions;
	SessionManager * manager;

	NodeMap found;

};

class HierarchySessionSupplier : public SessionSupplier {

public:
	void findNodes();
	PEditor createEditor(PSession session);

};

class CanvasSessionSupplier : public SessionSupplier {

public:
	void findNodes();
	PEditor createEditor(PSession session);
	PCanvasEditor getEditor();

	bool startPasting(const Glib::ustring & data, bool normal);
	void stopPasting();

	typedef sigc::signal<void> SignalPastingFinished;
	typedef sigc::signal<void, Glib::ustring> SignalStatusChanged;

	sigc::connection connect_pasting_finished(SignalPastingFinished::slot_type slot) { return signalPastingFinished.connect(slot); }
	sigc::connection connect_status_changed(SignalStatusChanged::slot_type slot) { return signalStatusChanged.connect(slot); }

protected:
	void pastingFinished() { signalPastingFinished.emit(); }
	void statusChanged(const Glib::ustring & text) { signalStatusChanged.emit(text); }

private:
	SignalPastingFinished signalPastingFinished;
	SignalStatusChanged signalStatusChanged;
	PNode current;

};

class SessionSupplierPropertiesBase : public SessionSupplier {

protected:
	void findProperties(PNode cut, PNode node);
	void screenNodes();
	bool mergeable(const Nodes & nodes);
	PEditor createPropertyEditor(PSession session);

};

class PropertiesSessionSupplier : public SessionSupplierPropertiesBase {

public:
	void findNodes();
	PEditor createEditor(PSession session);

};

class SignalsSessionSupplier : public SessionSupplier {

public:
	void findNodes();
	PEditor createEditor(PSession session);

};

class PackingSessionSupplier : public SessionSupplierPropertiesBase {

public:
	void findNodes();
	PEditor createEditor(PSession session);

};

class SessionManager {

public:
	SessionManager();
	~SessionManager();

	void addSupplier(SessionSupplier * supplier);
	void acceptSuppliers();

	Controller * getController() { return &controller; }
	Model * getModel() { return controller.getModel(); }

	const Nodes * getSelection() { return &selection; }
	const Nodes * getReducedSelection() { return &reducedSelection; }
	bool isSelected(PNode node);
	PNode getSingleSelection();
	PNode raiseSelectionParent();
	PNode raiseSelectionMaster();

	void submitSelection(const Nodes &);
	void deleteSelection();

	void begin(UpdateMode um=umNormal);
	void rollback();
	void commit();
	void commit(const Nodes & sel);

	void undo();
	void redo();
	bool canUndo();
	bool canRedo();

	LoadResult load(const Glib::ustring & data);
	void clear();
	void pasteToRoot(const Glib::ustring & data, bool normal);
	Glib::ustring save();
	Glib::ustring copy();

	PSession findSession(PNode node);
	bool isPerforming() { return performing; }
	Container * getContainer(PNode node);
	bool isXYChild(PNode node);
	bool isIndexChild(PNode node);

	typedef Pool<PNode> States;
	States * getStates() { return &states; }

	void applySettings();
	void storeSettings();

	typedef sigc::signal<void> SignalBeforeSessionsUpdate;
	typedef sigc::signal<void> SignalAfterSessionsUpdate;
	typedef sigc::signal<void> SignalSelectionChanged;
	typedef sigc::signal<void> SignalActionStarted;
	typedef sigc::signal<void> SignalActionCompleted;

	sigc::connection connect_before_sessions_update(SignalBeforeSessionsUpdate::slot_type slot) { return signalBeforeSessionsUpdate.connect(slot); }
	sigc::connection connect_after_sessions_update(SignalAfterSessionsUpdate::slot_type slot) { return signalAfterSessionsUpdate.connect(slot); }
	sigc::connection connect_selection_changed(SignalSelectionChanged::slot_type slot) { return signalSelectionChanged.connect(slot); }
	sigc::connection connect_action_started(SignalActionStarted::slot_type slot) { return signalActionStarted.connect(slot); }
	sigc::connection connect_action_completed(SignalActionCompleted::slot_type slot) { return signalActionCompleted.connect(slot); }

protected:
	void beginAction();
	void endAction();

	void updateSessions();

	bool acceptSelection(const Nodes & nodes);
	void updateSelection(const Nodes & nodes, bool force=false);

private:
	typedef std::list<SessionSupplier *> Suppliers;
	Suppliers suppliers;
	Nodes selection, reducedSelection;
	Controller controller;
	bool performing;
	States states;

	SignalBeforeSessionsUpdate signalBeforeSessionsUpdate;
	SignalAfterSessionsUpdate signalAfterSessionsUpdate;
	SignalSelectionChanged signalSelectionChanged;
	SignalActionStarted signalActionStarted;
	SignalActionCompleted signalActionCompleted;

};

}

#endif

==> crow-2.17.1.tar.bz2: crow-2.17.1/crow/session.h <==
/*
 *  Crow, a GUI designer for GTK+
 *  Copyright (c) 2005-2010 Maxim Udushlivy
 *
 *  Permission is hereby granted, free of charge, to any person
 *  obtaining a copy of this software and associated documentation
 *  files (the "Software"), to deal in the Software without
 *  restriction, including without limitation the rights to use, copy,
 *  modify, merge, publish, distribute, sublicense, and/or sell copies
 *  of the Software, and to permit persons to whom the Software is
 *  furnished to do so, subject to the following conditions:
 *
 *  The above copyright notice and this permission notice shall be
 *  included in all copies or substantial portions of the Software.
 *
 *  The software is provided "as is", without warranty of any kind,
 *  express or implied, including but not limited to the warranties of
 *  merchantability, fitness for a particular purpose and
 *  noninfringement. In no event shall the authors or copyright
 *  holders be liable for any claim, damages or other liability,
 *  whether in an action of contract, tort or otherwise, arising from,
 *  out of or in connection with the software or the use or other
 *  dealings in the software.
 */

#ifndef CROW_SESSION_H
#define CROW_SESSION_H

#include "controller.h"

namespace Crow {

enum SessionOperation { soRemove, soInsert, soUpdate };
enum SessionRole { srNone, srScalar, srVector, srObject };

class Session : public Object {

friend class SessionSupplier;

public:
	static PSession create();

	typedef std::vector<Property *> Properties;
	static SessionRole getRoleForNodes(const Nodes & nodes);
	static SessionRole getRoleForNode(PNode node);
	static Properties findProperties(Controller * controller, const Nodes & nodes);
	static EditorId getEditorId(const Properties & properties);
	static TypeId getPropertyType(const Properties & properties);

	void init(const Nodes & nodes, const IdPath & path, SessionManager * manager);
	void done();

	PEditor getEditor();
	void setEditor(PEditor editor);
	SessionManager * getManager() { return manager; }

	SessionRole getRole() { return role; }
	bool isMultiple();
	std::string getName();

	bool modelEditable();
	bool canEdit();
	bool isModified();
	bool isTouched();
	bool hasToSave();
	bool canDefault();

	bool hasHidden() { return hasFlags(sfHidden); }
	bool hasNoUndo() { return hasFlags(sfNoUndo); }
	bool areWeakLinks() { return haveFlags(sfWeakLink); }
	bool hasLinkOnly() { return hasFlags(sfLinkOnly); }
	bool areTranslatable() { return haveFlags(sfTranslatable); }
	bool allowTopMasters() { return haveFlags(sfAllowTM); }
	bool isElement();
	bool hasMasterLink();
	bool hasVectorWithMasterLink();
	bool areEmptyLinks();
	TypeId getFinalType();
	TypeId getVectorType();
	Any getScalar();
	Glib::ustring getMeta();
	TypeId getObjectBaseType();
	PNode getLinkTarget();
	TypeId getEntityType();
	EditorId getEditorId();
	bool isFuzzy();

	void resetScalar();
	void resetVector();
	void resetObject();
	void setScalarOnly(Any value);
	void setScalarAndMeta(Any value, ustring_cr meta);
	void setObjectAsLink(PNode object);
	void setObjectAsEntity(TypeId type);

	Property * getProperty1();

	const Nodes * getNodes() { return &nodes; }
	PNode getNode1();

	SessionOperation getOperation() { return operation; }
	const IdPath * getPath() { return &path; }

protected:
	Session();

	bool hasFlags(StateFlags sf);
	bool haveFlags(StateFlags sf);

	Properties getProperties();

private:
	SessionRole role;
	IdPath path;
	Nodes nodes;
	PEditor editor;
	SessionManager * manager;
	SessionOperation operation;

};

class SessionSupplier : public sigc::trackable {

public:
	SessionSupplier();
	virtual ~SessionSupplier() { }

	void setManager(SessionManager * m) { manager= m; }

	void updateSessions();
	void removeSessions();

	void editorsLoad();

	void applySettings();
	void storeSettings();

	PSession findSession(const IdPath & path);
	PSession findSession(PNode node);

	typedef sigc::signal<void, PSession> SignalSessionEvent;
	typedef sigc::signal<void, PSession> SignalAfterLoad;
	typedef sigc::signal<void> SignalPopupMenu;

	sigc::connection connect_session_event(SignalSessionEvent::slot_type slot) { return signalSessionEvent.connect(slot); }
	sigc::connection connect_after_load(SignalAfterLoad::slot_type slot) { return signalAfterLoad.connect(slot); }
	sigc::connection connect_popup_menu(SignalPopupMenu::slot_type slot) { return signalPopupMenu.connect(slot); }

protected:
	SessionManager * getManager() { return manager; }
	const Sessions * getSessions() { return &sessions; }

	virtual void findNodes() { }

	void append(PNode cut, PNode node);
	void append(const IdPath & path, PNode node);
	PSession createSession(const Nodes & nodes, const IdPath & path);
	virtual PEditor createEditor(PSession session);
	void stopSession(PSession);

	void sessionEvent(PSession session) { signalSessionEvent.emit(session); }
	void afterLoad(PSession session) { signalAfterLoad.emit(session); }
	void popupMenu() { signalPopupMenu.emit(); }

	typedef std::map<IdPath, Nodes> NodeMap;
	NodeMap * getFound() { return &found; }

private:
	SignalSessionEvent signalSessionEvent;
	SignalAfterLoad signalAfterLoad;
	SignalPopupMenu signalPopupMenu;
	Sessions sessions;
	SessionManager * manager;

	NodeMap found;

};

class HierarchySessionSupplier : public SessionSupplier {

public:
	void findNodes();
	PEditor createEditor(PSession session);

};

class CanvasSessionSupplier : public SessionSupplier {

public:
	void findNodes();
	PEditor createEditor(PSession session);
	PCanvasEditor getEditor();

	bool startPasting(ustring_cr data, bool normal);
	void stopPasting();

	typedef sigc::signal<void> SignalPastingFinished;
	typedef sigc::signal<void, Glib::ustring> SignalStatusChanged;

	sigc::connection connect_pasting_finished(SignalPastingFinished::slot_type slot) { return signalPastingFinished.connect(slot); }
	sigc::connection connect_status_changed(SignalStatusChanged::slot_type slot) { return signalStatusChanged.connect(slot); }

protected:
	void pastingFinished() { signalPastingFinished.emit(); }
	void statusChanged(ustring_cr text) { signalStatusChanged.emit(text); }

private:
	SignalPastingFinished signalPastingFinished;
	SignalStatusChanged signalStatusChanged;
	PNode current;

};

class SessionSupplierPropertiesBase : public SessionSupplier {

protected:
	void findProperties(PNode cut, PNode node);
	void screenNodes();
	bool mergeable(const Nodes & nodes);
	PEditor createPropertyEditor(PSession session);

};

class PropertiesSessionSupplier : public SessionSupplierPropertiesBase {

public:
	void findNodes();
	PEditor createEditor(PSession session);

};

class SignalsSessionSupplier : public SessionSupplier {

public:
	void findNodes();
	PEditor createEditor(PSession session);

};

class PackingSessionSupplier : public SessionSupplierPropertiesBase {

public:
	void findNodes();
	PEditor createEditor(PSession session);

};

class SessionManager {

public:
	SessionManager();
	~SessionManager();

	void addSupplier(SessionSupplier * supplier);
	void acceptSuppliers();

	Controller * getController() { return &controller; }
	Model * getModel() { return controller.getModel(); }

	const Nodes * getSelection() { return &selection; }
	const Nodes * getReducedSelection() { return &reducedSelection; }
	bool isSelected(PNode node);
	PNode getSingleSelection();
	PNode raiseSelectionParent();
	PNode raiseSelectionMaster();

	void submitSelection(const Nodes &);
	void deleteSelection();

	void begin(UpdateMode um=umNormal);
	void rollback();
	void commit();
	void commit(const Nodes & sel);

	void undo();
	void redo();
	bool canUndo();
	bool canRedo();

	LoadResult load(ustring_cr data);
	void clear();
	void pasteToRoot(ustring_cr data, bool normal);
	Glib::ustring save();
	Glib::ustring copy();

	PSession findSession(PNode node);
	bool isPerforming() { return performing; }
	Container * getContainer(PNode node);
	bool isXYChild(PNode node);
	bool isIndexChild(PNode node);

	typedef Pool<PNode> States;
	States * getStates() { return &states; }

	void applySettings();
	void storeSettings();

	typedef sigc::signal<void> SignalBeforeSessionsUpdate;
	typedef sigc::signal<void> SignalAfterSessionsUpdate;
	typedef sigc::signal<void> SignalSelectionChanged;
	typedef sigc::signal<void> SignalActionStarted;
	typedef sigc::signal<void> SignalActionCompleted;

	sigc::connection connect_before_sessions_update(SignalBeforeSessionsUpdate::slot_type slot) { return signalBeforeSessionsUpdate.connect(slot); }
	sigc::connection connect_after_sessions_update(SignalAfterSessionsUpdate::slot_type slot) { return signalAfterSessionsUpdate.connect(slot); }
	sigc::connection connect_selection_changed(SignalSelectionChanged::slot_type slot) { return signalSelectionChanged.connect(slot); }
	sigc::connection connect_action_started(SignalActionStarted::slot_type slot) { return signalActionStarted.connect(slot); }
	sigc::connection connect_action_completed(SignalActionCompleted::slot_type slot) { return signalActionCompleted.connect(slot); }

protected:
	void beginAction();
	void endAction();

	void updateSessions();

	bool acceptSelection(const Nodes & nodes);
	void updateSelection(const Nodes & nodes, bool force=false);

private:
	typedef std::list<SessionSupplier *> Suppliers;
	Suppliers suppliers;
	Nodes selection, reducedSelection;
	Controller controller;
	bool performing;
	States states;

	SignalBeforeSessionsUpdate signalBeforeSessionsUpdate;
	SignalAfterSessionsUpdate signalAfterSessionsUpdate;
	SignalSelectionChanged signalSelectionChanged;
	SignalActionStarted signalActionStarted;
	SignalActionCompleted signalActionCompleted;

};

}

#endif
